@@ -1,7 +1,7 @@
-/* $Id: actions.c,v 1.4 2011/08/09 11:38:34 imilh Exp $ */
+/* $Id: actions.c,v 1.34 2011/10/15 22:24:51 imilh Exp $ */
 
 /*
- * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010, 2011 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -31,84 +31,108 @@
  */
 
 #include "pkgin.h"
+#include <time.h>
 
 #ifndef LOCALBASE
 #define LOCALBASE "/usr/pkg" /* see DISCLAIMER below */
 #endif
 
 const char			*pkgin_cache = PKGIN_CACHE;
-static int			upgrade_type = UPGRADE_NONE;
-static uint8_t		said = 0;	
+static int			upgrade_type = UPGRADE_NONE, warn_count = 0, err_count = 0;
+static uint8_t		said = 0;
+FILE				*err_fp = NULL;
+long int			rm_filepos = -1, in_filepos = -1;
 
 int
-check_yesno(void)
+check_yesno(uint8_t default_answer)
 {
-	int c, r = 0;
+	const struct Answer	{
+		const uint8_t	numval;
+		const char		charval;
+	} answer[] = { { ANSW_NO, 'n' }, { ANSW_YES, 'y' } };
+
+	uint8_t			r, reverse_answer;
+	int				c;
 
 	if (yesflag)
-		return 1;
+		return ANSW_YES;
 	else if (noflag)
-		return 0;
+		return ANSW_NO;
 
-	printf(MSG_PROCEED);
+	reverse_answer = (default_answer == ANSW_YES) ? ANSW_NO : ANSW_YES;
 
-	if ((c = getchar()) == 'y')
-	    	r = 1;
-	else if (c == '\n' || c == EOF)
-	    	return 0;
+	if (default_answer == answer[ANSW_YES].numval)
+		printf(MSG_PROCEED_YES);
+	else
+		printf(MSG_PROCEED_NO);
+
+	if ((c = getchar()) == answer[reverse_answer].charval)
+		r = answer[reverse_answer].numval;
+	else
+		r = answer[default_answer].numval;
 
 	/* avoid residual char */
-	while((c = getchar()) != '\n' && c != EOF)
-		continue;
+	if (c != '\n')
+		while((c = getchar()) != '\n' && c != EOF)
+			continue;
+
 	return r;
 }
 
 static void
-pkg_download(Deptreehead *installhead)
+pkg_download(Plisthead *installhead)
 {
 	FILE		*fp;
-	Pkgdeptree	*pinstall;
+	Pkglist  	*pinstall;
 	struct stat	st;
 	Dlfile		*dlpkg;
-	char		pkg[BUFSIZ], query[BUFSIZ];
+	char		pkg_fs[BUFSIZ], pkg_url[BUFSIZ], query[BUFSIZ];
 
 	printf(MSG_DOWNLOAD_PKGS);
 
 	SLIST_FOREACH(pinstall, installhead, next) {
-		snprintf(pkg, BUFSIZ,
-		    "%s/%s%s", pkgin_cache, pinstall->depname, PKG_EXT);
+		snprintf(pkg_fs, BUFSIZ,
+		    "%s/%s%s", pkgin_cache, pinstall->depend, PKG_EXT);
 
 		/* pkg_info -X -a produces pkg_summary with empty FILE_SIZE,
 		 * people could spend some time blaming on pkgin before finding
 		 * what's really going on.
 		 */
 		if (pinstall->file_size == 0)
-			printf(MSG_EMPTY_FILE_SIZE, pinstall->depname);
+			printf(MSG_EMPTY_FILE_SIZE, pinstall->depend);
 
 		/* already fully downloaded */
-		if (stat(pkg, &st) == 0 && 
+		if (stat(pkg_fs, &st) == 0 && 
 			st.st_size == pinstall->file_size &&
 			pinstall->file_size != 0 )
 		    	continue;
 
-		umask(DEF_UMASK);
-		if ((fp = fopen(pkg, "w")) == NULL)
-			err(EXIT_FAILURE, MSG_ERR_OPEN, pkg);
-
-		snprintf(query, BUFSIZ, PKG_URL, pinstall->depname);
+		snprintf(query, BUFSIZ, PKG_URL, pinstall->depend);
 		/* retrieve repository for package  */
-		if (pkgindb_doquery(query, pdb_get_value, pkg) != 0)
-			errx(EXIT_FAILURE, MSG_PKG_NO_REPO, pinstall->depname);
+		if (pkgindb_doquery(query, pdb_get_value, pkg_url) != 0)
+			errx(EXIT_FAILURE, MSG_PKG_NO_REPO, pinstall->depend);
 
-		strlcat(pkg, "/", sizeof(pkg));
-		strlcat(pkg, pinstall->depname, sizeof(pkg));
-		strlcat(pkg, PKG_EXT, sizeof(pkg));
+		strlcat(pkg_url, "/", sizeof(pkg_url));
+		strlcat(pkg_url, pinstall->depend, sizeof(pkg_url));
+		strlcat(pkg_url, PKG_EXT, sizeof(pkg_url));
 
-		if ((dlpkg = download_file(pkg, NULL)) == NULL) {
-			fprintf(stderr, MSG_PKG_NOT_AVAIL, pinstall->depname);
-			if (!check_yesno())
+		/* if pkg's repo URL is file://, just symlink */
+		if (strncmp(pkg_url, SCHEME_FILE, strlen(SCHEME_FILE)) == 0) {
+			if (symlink(&pkg_url[strlen(SCHEME_FILE) + 3], pkg_fs) < 0)
+				errx(EXIT_FAILURE, MSG_SYMLINK_FAILED, pkg_fs);
+			printf(MSG_SYMLINKING_PKG, pkg_url);
+			continue;
+		}
+
+		umask(DEF_UMASK);
+		if ((fp = fopen(pkg_fs, "w")) == NULL)
+			err(EXIT_FAILURE, MSG_ERR_OPEN, pkg_fs);
+
+		if ((dlpkg = download_file(pkg_url, NULL)) == NULL) {
+			fprintf(stderr, MSG_PKG_NOT_AVAIL, pinstall->depend);
+			if (!check_yesno(DEFAULT_NO))
 				errx(EXIT_FAILURE, MSG_PKG_NOT_AVAIL,
-				    pinstall->depname);
+				    pinstall->depend);
 			pinstall->file_size = -1;
 			fclose(fp);
 			continue;
@@ -124,65 +148,150 @@ pkg_download(Deptreehead *installhead)
 
 }
 
-/* package removal */
+/**
+ * \brief Analyse PKG_INSTALL_ERR_LOG for warnings
+ */
 static void
-do_pkg_remove(Deptreehead *removehead)
+analyse_pkglog(long int filepos)
 {
-	Pkgdeptree *premove;
+	FILE		*err_ro;
+	char		err_line[BUFSIZ];
 
-/* send pkg_delete stderr to logfile */
-#ifdef HAVE_FREOPEN
+	if (filepos < 0)
+		return;
+
+	err_ro = fopen(PKG_INSTALL_ERR_LOG, "r");
+
+	(void)fseek(err_ro, filepos, SEEK_SET);
+
+	while (fgets(err_line, BUFSIZ, err_ro) != NULL) {
+		/* Warning: [...] was built for a platform */
+		if (strstr(err_line, "Warning") != NULL)
+			warn_count++;
+		/* A different version [...] is already installed */
+		if (strstr(err_line, "already installed") != NULL)
+			err_count--;
+		/* 1 package addition failed */
+		if (strstr(err_line, "addition failed") != NULL)
+			err_count++;
+		/* Can't install dependency */
+		if (strstr(err_line, "an\'t install") != NULL)
+			err_count++;
+	}
+
+	fclose(err_ro);
+}
+
+/**
+ * \brief Tags PKG_INSTALL_ERR_LOG with date
+ */
+#define DATELEN 64
+
+#ifndef DEBUG
+static void
+log_tag(const char *fmt, ...)
+{
+	va_list		ap;
+	char		log_action[BUFSIZ], now_date[DATELEN];
+	struct tm	tim;
+	time_t		now;
+
+	now = time(NULL);
+	tim = *(localtime(&now));
+
+	va_start(ap, fmt);
+	vsnprintf(log_action, BUFSIZ, fmt, ap);
+	va_end(ap);
+
+	(void)strftime(now_date, DATELEN, "%b %d %H:%M:%S", &tim);
+
+	fprintf(err_fp, "---%s: %s", now_date, log_action);
+	fflush(err_fp);
+	
+}
+#endif
+
+static void
+open_pi_log(void)
+{
 	if (!verbosity && !said) {
-		(void)freopen(PKGIN_ERR_LOG, "a", stderr);
-		printf(MSG_LOGGING_TO, PKGIN_ERR_LOG);
+		if ((err_fp = fopen(PKG_INSTALL_ERR_LOG, "a")) == NULL) {
+ 			fprintf(stderr, MSG_CANT_OPEN_WRITE, PKG_INSTALL_ERR_LOG);
+			exit(EXIT_FAILURE);
+		}
+
+		dup2(fileno(err_fp), STDERR_FILENO);
+
+		rm_filepos = ftell(err_fp);
 		said = 1;
 	}
-#endif
+}
+
+static void
+close_pi_log(void)
+{
+	if (!verbosity) {
+		analyse_pkglog(rm_filepos);
+		printf(MSG_WARNS_ERRS, warn_count, err_count);
+		if (warn_count > 0 || err_count > 0)
+			printf(MSG_PKG_INSTALL_LOGGING_TO, PKG_INSTALL_ERR_LOG);
+	}
+}
+
+/* package removal */
+void
+do_pkg_remove(Plisthead *removehead)
+{
+	Pkglist *premove;
+
+	/* send pkg_delete stderr to logfile */
+	open_pi_log();
 
 	SLIST_FOREACH(premove, removehead, next) {
 		/* file not available in the repository */
 		if (premove->file_size == -1)
 			continue;
 
-		if (premove->depname == NULL)
+		if (premove->depend == NULL)
 			/* SLIST corruption, badly installed package */
 			continue;
 
 		/* pkg_install cannot be deleted */
-		if (strncmp(premove->depname, PKG_INSTALL, strlen(PKG_INSTALL)) == 0) {
+		if (strcmp(premove->depend, PKG_INSTALL) == 0) {
 			printf(MSG_NOT_REMOVING, PKG_INSTALL);
 			continue;
 		}
 
-		printf(MSG_REMOVING, premove->depname);
-		fprintf(stderr, "%s %s %s\n",
-			PKG_DELETE, pkgtools_flags, premove->depname);
+		printf(MSG_REMOVING, premove->depend);
 #ifndef DEBUG
-		fexec(PKG_DELETE, pkgtools_flags, premove->depname, NULL);
+		if (!verbosity)
+			log_tag(MSG_REMOVING, premove->depend);
+		if (fexec(PKG_DELETE, pkgtools_flags, premove->depend, NULL)
+			!= EXIT_SUCCESS)
+			err_count++;
 #endif
 	}
+
+	close_pi_log();
 }
 
-/* package installation. Don't rely on pkg_add's ability to fetch and
+/**
+ * \fn do_pkg_install
+ *
+ * package installation. Don't rely on pkg_add's ability to fetch and
  * install as we want to keep control on packages installation order.
  * Besides, pkg_add cannot be used to install an "older" package remotely
  * i.e. apache 1.3
  */
 static void
-do_pkg_install(Deptreehead *installhead)
+do_pkg_install(Plisthead *installhead)
 {
-	Pkgdeptree	*pinstall;
+	Pkglist		*pinstall;
 	char		pkgpath[BUFSIZ];
 	char		pi_tmp_flags[5]; /* tmp force flags for pkg_install */
 
-/* send pkg_add stderr to logfile */
-#ifdef HAVE_FREOPEN
-	if (!verbosity && !said) {
-		(void)freopen(PKGIN_ERR_LOG, "a", stderr);
-		printf(MSG_LOGGING_TO, PKGIN_ERR_LOG);
-		said = 1;
-	}
-#endif
+	/* send pkg_add stderr to logfile */
+	open_pi_log();
 
 	printf(MSG_INSTALL_PKG);
 
@@ -192,21 +301,25 @@ do_pkg_install(Deptreehead *installhead)
 		if (pinstall->file_size == -1)
 			continue;
 
-		printf(MSG_INSTALLING, pinstall->depname);
+		printf(MSG_INSTALLING, pinstall->depend);
 		snprintf(pkgpath, BUFSIZ,
-			"%s/%s%s", pkgin_cache, pinstall->depname, PKG_EXT);
+			"%s/%s%s", pkgin_cache, pinstall->depend, PKG_EXT);
+
+#ifndef DEBUG
+		if (!verbosity)
+			log_tag(MSG_INSTALLING, pinstall->depend);
+#endif
 
 		/* are we upgrading pkg_install ? */
-		if (strncmp(pinstall->depname, PKG_INSTALL,
-				strlen(PKG_INSTALL)) == 0) {
+		if (pi_upgrade) { /* set in order.c */
+			pi_upgrade = 0; /* 1st item on the list, reset the flag */
 			printf(MSG_UPGRADE_PKG_INSTALL, PKG_INSTALL);
 			/* set temporary force flags */
 			strncpy(pi_tmp_flags, "-ffu", 5);
 			if (verbosity)
 				/* append verbosity if requested */
 				strncat(pi_tmp_flags, "v", 2);
-			if (check_yesno()) {
-				fprintf(stderr, "%s %s %s\n", PKG_ADD, pi_tmp_flags, pkgpath);
+			if (check_yesno(DEFAULT_YES)) {
 #ifndef DEBUG
 				fexec(PKG_ADD, pi_tmp_flags, pkgpath, NULL);
 #endif
@@ -214,12 +327,13 @@ do_pkg_install(Deptreehead *installhead)
 				continue;
 		} else {
 			/* every other package */
-			fprintf(stderr, "%s %s %s\n", PKG_ADD, pkgtools_flags, pkgpath);
 #ifndef DEBUG
 			fexec(PKG_ADD, pkgtools_flags, pkgpath, NULL);
 #endif
 		}
 	} /* installation loop */
+
+	close_pi_log();
 }
 
 /* build the output line */
@@ -244,206 +358,53 @@ action_list(char *flatlist, char *str)
 	return newlist;
 }
 
-/* find required files (REQUIRES) from PROVIDES or filename */
-static int
-pkg_met_reqs(Impacthead *impacthead)
-{
-	int			met_reqs = 1, foundreq;
-	Pkgimpact	*pimpact;
-	Plisthead	*requireshead;
-	Pkglist		*requires;
-#ifdef CHECK_PROVIDES
-	Pkgimpact	*impactprov;
-	Plisthead	*provideshead;
-	Pkglist		*provides;
-#endif
-	struct stat	sb;
-	char		query[BUFSIZ];
-
-	/* first, parse impact list */
-	SLIST_FOREACH(pimpact, impacthead, next) {
-		/* retreive requires list for package */
-		snprintf(query, BUFSIZ, GET_REQUIRES_QUERY, pimpact->pkgname);
-		requireshead = rec_pkglist(query);
-
-		if (requireshead == NULL) /* empty requires list (very unlikely) */
-			continue;
-
-		/* parse requires list */
-		SLIST_FOREACH(requires, requireshead, next) {
-
-			foundreq = 0;
-
-			/* for performance sake, first check basesys */
-			if ((strncmp(requires->pkgname, LOCALBASE,
-				    sizeof(LOCALBASE) - 1)) != 0) {
-				if (stat(requires->pkgname, &sb) < 0) {
-					printf(MSG_REQT_NOT_PRESENT,
-						requires->pkgname, pimpact->pkgname);
-
-					met_reqs = 0;
-				}
-				/* was a basysfile, no need to check PROVIDES */
-				continue;
-			}
-			/* FIXME: the code below actually works, but there's no
-			 * point losing performances when some REQUIRES do not match
-			 * PROVIDES in pkg_summary(5). This is a known issue and will
-			 * hopefuly be fixed.
-			 */
-#ifndef CHECK_PROVIDES
-			continue;
-#else
-			/* search what local packages provide */
-			provideshead = rec_pkglist(LOCAL_PROVIDES);
-			SLIST_FOREACH(provides, provideshead, next) {
-				if (strncmp(provides->pkgname,
-						requires->pkgname,
-						strlen(requires->pkgname)) == 0) {
-
-					foundreq = 1;
-
-					/* found, no need to go further*/
-					break;
-				} /* match */
-			} /* SLIST_FOREACH LOCAL_PROVIDES */
-			free_pkglist(provideshead);
-
-			/* REQUIRES was not found on local packages, try impact list */
-			if (!foundreq) {
-				/* re-parse impact list to retreive PROVIDES */
-				SLIST_FOREACH(impactprov, impacthead, next) {
-					snprintf(query, BUFSIZ, GET_PROVIDES_QUERY,
-						impactprov->pkgname);
-					provideshead = rec_pkglist(query);
-
-					if (provideshead == NULL)
-						continue;
-
-					/* then parse provides list for every package */
-					SLIST_FOREACH(provides, provideshead, next) {
-						if (strncmp(provides->pkgname,
-								requires->pkgname,
-								strlen(requires->pkgname)) == 0) {
-
-							foundreq = 1;
-
-							/* found, no need to go further
-							   return to impactprov list */
-							break;
-						} /* match */
-					}
-					free_pkglist(provideshead);
-
-					if (foundreq) /* exit impactprov list loop */
-						break;
-
-				} /* SLIST_NEXT impactprov */
-
-			} /* if (!foundreq) LOCAL_PROVIDES -> impact list */
-
-			/* FIXME: BIG FAT DISCLAIMER
-			 * as of 04/2009, some packages described in pkg_summary
-			 * have unmet REQUIRES. This is a known bug that makes the
-			 * PROVIDES untrustable and some packages uninstallable.
-			 * foundreq is forced to 1 for now for every REQUIRES
-			 * matching LOCALBASE, which is hardcoded to "/usr/pkg"
-			 */
-			if (!foundreq) {
-				printf(MSG_REQT_NOT_PRESENT_BUT, requires->pkgname);
-
-				foundreq = 1;
-			}
-#endif
-		} /* SLIST_FOREACH requires */
-		free_pkglist(requireshead);
-	} /* 1st impact SLIST_FOREACH */
-
-	return met_reqs;
-}
-
-/* check for conflicts and if needed files are present */
-static int
-pkg_has_conflicts(Plisthead *conflictshead, Pkgimpact *pimpact)
-{
-	int			has_conflicts = 0;
-	Pkglist		*conflicts; /* SLIST conflicts pointer */
-	char		*conflict_pkg, query[BUFSIZ];
-
-	if (conflictshead == NULL)
-		return 0;
-
-	/* check conflicts */
-	SLIST_FOREACH(conflicts, conflictshead, next) {
-		if (pkg_match(conflicts->pkgname, pimpact->pkgname)) {
-
-			/* got a conflict, retrieve conflicting local package */
-			snprintf(query, BUFSIZ,
-				GET_CONFLICT_QUERY, conflicts->pkgname);
-
-			XMALLOC(conflict_pkg, BUFSIZ * sizeof(char));
-			if (pkgindb_doquery(query,
-					pdb_get_value, conflict_pkg) == 0)
-
-				printf(MSG_CONFLICT_PKG,
-					pimpact->pkgname, conflict_pkg);
-
-			XFREE(conflict_pkg);
-
-			has_conflicts = 1;
-		} /* match conflict */
-	} /* SLIST_FOREACH conflicts */
-
-	return has_conflicts;
-}
-
 #define H_BUF 6
 
 int
-pkgin_install(char **pkgargs, uint8_t do_inst)
+pkgin_install(char **opkgargs, uint8_t do_inst)
 {
 	int			installnum = 0, upgradenum = 0, removenum = 0;
 	int			rc = EXIT_FAILURE;
 	uint64_t   	file_size = 0, size_pkg = 0;
-	Impacthead	*impacthead; /* impact head */
-	Pkgimpact	*pimpact;
-	Deptreehead	*removehead = NULL, *installhead = NULL;
-	Pkgdeptree	*premove, *pinstall; /* not a Pkgdeptree, just for ease */
-	Plisthead	*conflictshead; /* conflicts head */
+	Pkglist		*premove, *pinstall;
+	Pkglist		*pimpact;
+	Plisthead	*impacthead; /* impact head */
+	Plisthead	*removehead = NULL, *installhead = NULL;
+	char		**pkgargs;
 	char		*toinstall = NULL, *toupgrade = NULL, *toremove = NULL;
+	char		*unmet_reqs = NULL;
 	char		pkgpath[BUFSIZ], h_psize[H_BUF], h_fsize[H_BUF];
 	struct stat	st;
 
-	/* full impact list */
-	if ((impacthead = pkg_impact(pkgargs)) == NULL) {
+	/* transform command line globs into pkgnames */
+	if ((pkgargs = glob_to_pkgarg(opkgargs)) == NULL) {
 		printf(MSG_NOTHING_TO_DO);
 		return rc;
 	}
 
-	/* check for required files */
-	if (!pkg_met_reqs(impacthead)) {
-		printf(MSG_REQT_MISSING);
+	/* full impact list */
+	if ((impacthead = pkg_impact(pkgargs)) == NULL) {
+		printf(MSG_NOTHING_TO_DO);
+		free_list(pkgargs);
 		return rc;
 	}
 
-	/* conflicts list */
-	conflictshead = rec_pkglist(LOCAL_CONFLICTS);
+	/* check for required files */
+	if (!pkg_met_reqs(impacthead))
+		SLIST_FOREACH(pimpact, impacthead, next)
+			if (pimpact->action == UNMET_REQ)
+				unmet_reqs = action_list(unmet_reqs, pimpact->full);
 
 	/* browse impact tree */
 	SLIST_FOREACH(pimpact, impacthead, next) {
 
 		/* check for conflicts */
-		if (pkg_has_conflicts(conflictshead, pimpact)) {
-			if (!check_yesno()) {
-				free_impact(impacthead);
-				XFREE(impacthead);
-
-				return rc;
-			}
-		}
+		if (pkg_has_conflicts(pimpact))
+			if (!check_yesno(DEFAULT_NO))
+				goto installend;
 
 		snprintf(pkgpath, BUFSIZ, "%s/%s%s",
-			pkgin_cache, pimpact->pkgname, PKG_EXT);
+			pkgin_cache, pimpact->full, PKG_EXT);
 
 		/* if package is not already downloaded or size mismatch, d/l it */
 		if (stat(pkgpath, &st) < 0 || st.st_size != pimpact->file_size)
@@ -466,8 +427,6 @@ pkgin_install(char **pkgargs, uint8_t do_inst)
 			break;
 		}
 	}
-	/* free conflicts list */
-	free_pkglist(conflictshead);
 
 	(void)humanize_number(h_fsize, H_BUF, (int64_t)file_size, "",
 		HN_AUTOSCALE, HN_B | HN_NOSPACE | HN_DECIMAL);
@@ -487,19 +446,35 @@ pkgin_install(char **pkgargs, uint8_t do_inst)
 		removehead = order_upgrade_remove(impacthead);
 
 		SLIST_FOREACH(premove, removehead, next) {
-			if (premove->computed == TOUPGRADE)
-				toupgrade = action_list(toupgrade, premove->depname);
+			if (premove->computed == TOUPGRADE) {
+				toupgrade = action_list(toupgrade, premove->depend);
+#ifdef DEBUG
+				printf("package: %s - level: %d\n",
+					premove->depend, premove->level);
+#endif
+			}
 		}
 		printf(MSG_PKGS_TO_UPGRADE, upgradenum, toupgrade);
 		printf("\n");
 
 		if (removenum > 0) {
 			SLIST_FOREACH(premove, removehead, next) {
-				if (premove->computed == TOREMOVE)
-					toremove = action_list(toremove, premove->depname);
+				if (premove->computed == TOREMOVE) {
+					toremove = action_list(toremove, premove->depend);
+#ifdef DEBUG
+					printf("package: %s - level: %d\n",
+						premove->depend, premove->level);
+#endif
+				}
+			}
+			/* 
+			 * some packages may have been marked as TOREMOVE, then 
+			 * discovered as TOUPGRADE
+			 */
+			if (toremove != NULL) {
+				printf(MSG_PKGS_TO_REMOVE, removenum, toremove);
+				printf("\n");
 			}
-			printf(MSG_PKGS_TO_REMOVE, removenum, toremove);
-			printf("\n");
 		}
 
 	} else
@@ -509,13 +484,21 @@ pkgin_install(char **pkgargs, uint8_t do_inst)
 		/* record ordered install list */
 		installhead = order_install(impacthead);
 
-		SLIST_FOREACH(pinstall, installhead, next)
-			toinstall = action_list(toinstall, pinstall->depname);
+		SLIST_FOREACH(pinstall, installhead, next) {
+			toinstall = action_list(toinstall, pinstall->depend);
+#ifdef DEBUG
+			printf("package: %s - level: %d\n",
+				pinstall->depend, pinstall->level);
+#endif
+		}
 
 		printf(MSG_PKGS_TO_INSTALL, installnum, toinstall, h_fsize, h_psize);
 		printf("\n");
 
-		if (check_yesno()) {
+		if (unmet_reqs != NULL)/* there were unmet requirements */
+			printf(MSG_REQT_MISSING, unmet_reqs);
+
+		if (check_yesno(DEFAULT_YES)) {
 			/* before erasing anything, download packages */
 			pkg_download(installhead);
 
@@ -534,19 +517,19 @@ pkgin_install(char **pkgargs, uint8_t do_inst)
 				
 				rc = EXIT_SUCCESS;
 			}
-		}
+		} /* check_yesno */
+
 	} else
 		printf(MSG_NOTHING_TO_INSTALL);
 
+installend:
 
 	XFREE(toinstall);
 	XFREE(toupgrade);
-	free_impact(impacthead);
-	XFREE(impacthead);
-	free_deptree(removehead);
-	XFREE(removehead);
-	free_deptree(installhead);
-	XFREE(installhead);
+	free_pkglist(&impacthead, IMPACT);
+	free_pkglist(&removehead, DEPTREE);
+	free_pkglist(&installhead, DEPTREE);
+	free_list(pkgargs);
 
 	return rc;
 }
@@ -555,22 +538,19 @@ int
 pkgin_remove(char **pkgargs)
 {
 	int			deletenum = 0, exists, rc;
-	Deptreehead	pdphead, *removehead;
-	Pkgdeptree	*pdp;
-	Plisthead	*plisthead;
+	Plisthead	*pdphead, *removehead;
+	Pkglist		*pdp;
 	char   		*todelete = NULL, **ppkgargs, *pkgname, *ppkg;
 
-	SLIST_INIT(&pdphead);
+	pdphead = init_head();
 
-	plisthead = rec_pkglist(LOCAL_PKGS_QUERY);
-
-	if (plisthead == NULL)
+	if (SLIST_EMPTY(&l_plisthead))
 		errx(EXIT_FAILURE, MSG_EMPTY_LOCAL_PKGLIST);
 
 	/* act on every package passed to the command line */
 	for (ppkgargs = pkgargs; *ppkgargs != NULL; ppkgargs++) {
 
-		if ((pkgname = find_exact_pkg(plisthead, *ppkgargs)) == NULL) {
+		if ((pkgname = find_exact_pkg(&l_plisthead, *ppkgargs)) == NULL) {
 			printf(MSG_PKG_NOT_INSTALLED, *ppkgargs);
 			continue;
 		}
@@ -578,15 +558,15 @@ pkgin_remove(char **pkgargs)
 		trunc_str(ppkg, '-', STR_BACKWARD);
 
 		/* record full reverse dependency list for package */
-		full_dep_tree(ppkg, LOCAL_REVERSE_DEPS, &pdphead);
+		full_dep_tree(ppkg, LOCAL_REVERSE_DEPS, pdphead);
 
 		XFREE(ppkg);
 
 		exists = 0;
 		/* check if package have already been recorded */
-		SLIST_FOREACH(pdp, &pdphead, next) {
-			if (strncmp(pdp->depname, pkgname,
-					strlen(pdp->depname)) == 0) {
+		SLIST_FOREACH(pdp, pdphead, next) {
+			if (strncmp(pdp->depend, pkgname,
+					strlen(pdp->depend)) == 0) {
 				exists = 1;
 				break;
 			}
@@ -598,34 +578,33 @@ pkgin_remove(char **pkgargs)
 		}
 
 		/* add package itself */
-		XMALLOC(pdp, sizeof(Pkgdeptree));
-		pdp->depname = pkgname;
+		pdp = malloc_pkglist(DEPTREE);
 
-		if (SLIST_EMPTY(&pdphead))
+		pdp->depend = pkgname;
+
+		if (SLIST_EMPTY(pdphead))
 			/* identify unique package, don't cut it when ordering */
 			pdp->level = -1;
 		else
 			pdp->level = 0;
 
-		XSTRDUP(pdp->matchname, pdp->depname);
-		trunc_str(pdp->matchname, '-', STR_BACKWARD);
+		XSTRDUP(pdp->name, pdp->depend);
+		trunc_str(pdp->name, '-', STR_BACKWARD);
 
-		SLIST_INSERT_HEAD(&pdphead, pdp, next);
+		SLIST_INSERT_HEAD(pdphead, pdp, next);
 	} /* for pkgargs */
 
-	free_pkglist(plisthead);
-
 	/* order remove list */
-	removehead = order_remove(&pdphead);
+	removehead = order_remove(pdphead);
 
 	SLIST_FOREACH(pdp, removehead, next) {
 		deletenum++;
-		todelete = action_list(todelete, pdp->depname);
+		todelete = action_list(todelete, pdp->depend);
 	}
 
 	if (todelete != NULL) {
 		printf(MSG_PKGS_TO_DELETE, deletenum, todelete);
-		if (check_yesno()) {
+		if (check_yesno(DEFAULT_YES)) {
 			do_pkg_remove(removehead);
 
 			update_db(LOCAL_SUMMARY, NULL);
@@ -634,13 +613,15 @@ pkgin_remove(char **pkgargs)
 		} else
 			rc = EXIT_FAILURE;
 
+		analyse_pkglog(rm_filepos);
 	} else {
 		printf(MSG_NO_PKGS_TO_DELETE);
 		rc = EXIT_SUCCESS;
 	}
 
-	free_deptree(removehead);
-	XFREE(removehead);
+	free_pkglist(&removehead, DEPTREE);
+	free_pkglist(&pdphead, DEPTREE);
+
 	XFREE(todelete);
 
 	return rc;
@@ -648,35 +629,31 @@ pkgin_remove(char **pkgargs)
 
 /* find closest match for packages to be upgraded */
 static char *
-narrow_match(Plisthead *remoteplisthead,
-	char *pkgname, const char *fullpkgname)
+narrow_match(char *pkgname, const char *fullpkgname)
 {
 	Pkglist	*pkglist;
-	char	*rpkgname, *best_match = NULL;
-	int		pkglen, fullpkglen, i, matchlen = 0;
+	char	*best_match = NULL;
+	unsigned int		i;
+	size_t  pkglen, fullpkglen, matchlen;
 
+	matchlen = 0;
 	pkglen = strlen(pkgname);
 	fullpkglen = strlen(fullpkgname);
 
-	SLIST_FOREACH(pkglist, remoteplisthead, next) {
-		XSTRDUP(rpkgname, pkglist->pkgname);
-		trunc_str(rpkgname, '-', STR_BACKWARD);
-
-		if (strlen(rpkgname) == pkglen &&
-			strncmp(pkgname, rpkgname, pkglen) == 0) {
+	SLIST_FOREACH(pkglist, &r_plisthead, next) {
+		if (strlen(pkglist->name) == pkglen &&
+			strncmp(pkgname, pkglist->name, pkglen) == 0) {
 
 			for (i = 0;
-				 i < fullpkglen && fullpkgname[i] == pkglist->pkgname[i];
+				 i < fullpkglen && fullpkgname[i] == pkglist->full[i];
 				i++);
 
 			if (i > matchlen) {
 				matchlen = i;
-				XSTRDUP(best_match, pkglist->pkgname);
+				XSTRDUP(best_match, pkglist->full);
 			}
 
 		}
-
-		XFREE(rpkgname);
 	} /* SLIST_FOREACH remoteplisthead */
 	XFREE(pkgname);
 
@@ -684,7 +661,7 @@ narrow_match(Plisthead *remoteplisthead,
 }
 
 static char **
-record_upgrades(Plisthead *plisthead, Plisthead *remoteplisthead)
+record_upgrades(Plisthead *plisthead)
 {
 	Pkglist	*pkglist;
 	int		count = 0;
@@ -697,11 +674,9 @@ record_upgrades(Plisthead *plisthead, Plisthead *remoteplisthead)
 
 	count = 0;
 	SLIST_FOREACH(pkglist, plisthead, next) {
-		XSTRDUP(pkgargs[count], pkglist->pkgname);
-		trunc_str(pkgargs[count], '-', STR_BACKWARD);
+		XSTRDUP(pkgargs[count], pkglist->name);
 
-		pkgargs[count] = narrow_match(remoteplisthead,
-			pkgargs[count], pkglist->pkgname);
+		pkgargs[count] = narrow_match(pkgargs[count], pkglist->full);
 
 		if (pkgargs[count] == NULL)
 			continue;
@@ -716,7 +691,7 @@ record_upgrades(Plisthead *plisthead, Plisthead *remoteplisthead)
 void
 pkgin_upgrade(int uptype)
 {
-	Plisthead	*keeplisthead, *localplisthead, *remoteplisthead;
+	Plisthead	*keeplisthead, *localplisthead;
 	char		**pkgargs;
 
 	/* used for pkgin_install not to update database, this is done below */
@@ -728,16 +703,14 @@ pkgin_upgrade(int uptype)
 
 	/* upgrade all packages, not only keepables */
 	if (uptype == UPGRADE_ALL) {
-		if ((localplisthead = rec_pkglist(LOCAL_PKGS_QUERY)) == NULL)
+		if (SLIST_EMPTY(&l_plisthead))
 			errx(EXIT_FAILURE, MSG_EMPTY_LOCAL_PKGLIST);
+		localplisthead = &l_plisthead;
 	} else
 		/* upgrade only keepables packages */
 		localplisthead = keeplisthead;
 
-	if ((remoteplisthead = rec_pkglist(REMOTE_PKGS_QUERY)) == NULL)
-		errx(EXIT_FAILURE, MSG_EMPTY_AVAIL_PKGLIST);
-
-	pkgargs = record_upgrades(localplisthead, remoteplisthead);
+	pkgargs = record_upgrades(localplisthead);
 
 	if (pkgin_install(pkgargs, DO_INST) == EXIT_SUCCESS) {
 		/*
@@ -745,11 +718,9 @@ pkgin_upgrade(int uptype)
 		 * in order to restore them
 		 */
 		if (uptype == UPGRADE_ALL) {
-
-			free_pkglist(localplisthead);
 			free_list(pkgargs);
 			/* record keep list */
-			pkgargs = record_upgrades(keeplisthead, remoteplisthead);
+			pkgargs = record_upgrades(keeplisthead);
 		}
 
 		update_db(LOCAL_SUMMARY, pkgargs);
@@ -757,6 +728,5 @@ pkgin_upgrade(int uptype)
 
 	free_list(pkgargs);
 
-	free_pkglist(remoteplisthead);
-	free_pkglist(keeplisthead);
+	free_pkglist(&keeplisthead, LIST);
 }
