@@ -1,7 +1,7 @@
-/* $Id: main.c,v 1.1.1.1 2011/03/03 14:43:12 imilh Exp $ */
+/* $Id: main.c,v 1.17 2011/10/01 18:06:44 imilh Exp $ */
 
 /*
- * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010, 2011 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -35,19 +35,20 @@
 #include "lib.h"
 
 static void	usage(void);
-static void	split_repos(void);
 static int	find_cmd(const char *);
 static void	missing_param(int, int, const char *);
+static void ginto(void);
 
 uint8_t		yesflag = 0, noflag = 0, force_update = 0, force_reinstall = 0;
 uint8_t		verbosity = 0, package_version = 0;
-char		*env_repos, **pkg_repos;
 char		lslimit = '\0';
 char		pkgtools_flags[5];
+FILE  		*tracefp = NULL;
 
 int
 main(int argc, char *argv[])
 {
+	uint8_t		updb_all;
 	uint8_t		do_inst = DO_INST; /* by default, do install packages */
 	int 		ch;
 	struct stat	sb;
@@ -58,7 +59,7 @@ main(int argc, char *argv[])
 	if (argc < 2)
 		usage();
 
-	while ((ch = getopt(argc, argv, "dhyfFPvVl:nc:")) != -1) {
+	while ((ch = getopt(argc, argv, "dhyfFPvVl:nc:t:")) != -1) {
 		switch (ch) {
 		case 'f':
 			force_update = 1;
@@ -97,6 +98,10 @@ main(int argc, char *argv[])
 		case 'P':
 			package_version = 1;
 			break;
+		case 't':
+			if ((tracefp = fopen(optarg, "w")) == NULL)
+				err(EXIT_FAILURE, MSG_CANT_OPEN_WRITE, optarg);
+			break;
 		default:
 			usage();
 			/* NOTREACHED */
@@ -122,6 +127,10 @@ main(int argc, char *argv[])
 			errx(-1, MSG_CHDIR_FAILED);
 	}
 
+	/* check for pkg_install */
+	if (stat(PKG_ADD, &sb) < 0)
+		errx(EXIT_FAILURE, MSG_PKG_INSTALL_NOT_PRESENT);
+
 	/* for pkg_install */
 	unsetenv("PKG_PATH");
 	/* create base directories */
@@ -130,15 +139,29 @@ main(int argc, char *argv[])
 
 	pkgindb_init();
 
+	/* check if current database fits our needs */
+	updb_all = upgrade_database();
+
 	/* update local db if pkgdb mtime has changed */
 	update_db(LOCAL_SUMMARY, NULL);
 
 	/* split PKG_REPOS env variable and record them */
 	split_repos();
 
+	/* 
+	 * upgrade remote database if pkgin version changed and not compatible 
+	 * or if empty database
+	 */
+	if (updb_all)
+		update_db(REMOTE_SUMMARY, NULL);
+
 	/* find command index */
 	ch = find_cmd(argv[0]);
 
+	/* we need packages lists for almost everything */
+	if (ch != PKG_UPDT_CMD) /* already loaded by update_db() */
+		init_global_pkglists();
+
 	/* fill pkgtools flags */
 	if (verbosity)
 		strncpy(pkgtools_flags, "-fv", 3);
@@ -146,72 +169,95 @@ main(int argc, char *argv[])
 		strncpy(pkgtools_flags, "-f", 2);
 
 	switch (ch) {
-		case PKG_UPDT_CMD:
-			update_db(REMOTE_SUMMARY, NULL);
-			break;
-		case PKG_SHDDP_CMD:
-			missing_param(argc, 2, MSG_MISSING_PKGNAME);
-			show_direct_depends(argv[1]);
-			break;
-		case PKG_SHFDP_CMD:
-			missing_param(argc, 2, MSG_MISSING_PKGNAME);
-			show_full_dep_tree(argv[1],
-				DIRECT_DEPS, MSG_FULLDEPTREE);
-			break;
-		case PKG_SHRDP_CMD:
-			missing_param(argc, 2, MSG_MISSING_PKGNAME);
-			show_full_dep_tree(argv[1],
-				LOCAL_REVERSE_DEPS,
-				MSG_REVDEPTREE);
-			break;
-		case PKG_LLIST_CMD:
-			list_pkgs(LOCAL_PKGS_QUERY, PKG_LLIST_CMD);
-			break;
-		case PKG_RLIST_CMD:
-			list_pkgs(REMOTE_PKGS_QUERY, PKG_RLIST_CMD);
-			break;
-		case PKG_INST_CMD:
-			missing_param(argc, 2, MSG_PKG_ARGS_INST);
-			pkgin_install(&argv[1], do_inst);
-			break;
-		case PKG_UPGRD_CMD:
-			pkgin_upgrade(UPGRADE_KEEP);
-			break;
-		case PKG_FUPGRD_CMD:
-			pkgin_upgrade(UPGRADE_ALL);
-			break;
-		case PKG_REMV_CMD:
-			missing_param(argc, 2, MSG_PKG_ARGS_RM);
-			pkgin_remove(&argv[1]);
-			break;
-		case PKG_AUTORM_CMD:
-			pkgin_autoremove();
-			break;
-		case PKG_KEEP_CMD:
-			missing_param(argc, 2, MSG_PKG_ARGS_KEEP);
-			pkg_keep(KEEP, &argv[1]);
-			break;
-		case PKG_UNKEEP_CMD:
-			missing_param(argc, 2, MSG_PKG_ARGS_UNKEEP);
-			pkg_keep(UNKEEP, &argv[1]);
-			break;
-		case PKG_SHKP_CMD:
-			show_pkg_keep();
-			break;
-		case PKG_SRCH_CMD:
-			missing_param(argc, 2, MSG_MISSING_SRCH);
-			search_pkg(argv[1]);
-			break;
-		case PKG_CLEAN_CMD:
-			clean_cache();
-			break;
-		default:
-			usage();
-			/* NOTREACHED */
+	case PKG_UPDT_CMD: /* update packages db */
+		update_db(REMOTE_SUMMARY, NULL);
+		break;
+	case PKG_SHDDP_CMD: /* show direct depends */
+		missing_param(argc, 2, MSG_MISSING_PKGNAME);
+		show_direct_depends(argv[1]);
+		break;
+	case PKG_SHFDP_CMD: /* show full dependency tree */
+		missing_param(argc, 2, MSG_MISSING_PKGNAME);
+		show_full_dep_tree(argv[1],
+			DIRECT_DEPS, MSG_FULLDEPTREE);
+		break;
+	case PKG_SHRDP_CMD: /* show full reverse dependency tree */
+		missing_param(argc, 2, MSG_MISSING_PKGNAME);
+		show_full_dep_tree(argv[1],
+			LOCAL_REVERSE_DEPS,
+			MSG_REVDEPTREE);
+		break;
+	case PKG_LLIST_CMD:
+		list_pkgs(LOCAL_PKGS_QUERY, PKG_LLIST_CMD); /* list local packages */
+		break;
+	case PKG_RLIST_CMD: /* list available packages */
+		list_pkgs(REMOTE_PKGS_QUERY, PKG_RLIST_CMD);
+		break;
+	case PKG_INST_CMD: /* install a package and its dependencies */
+		missing_param(argc, 2, MSG_PKG_ARGS_INST);
+		pkgin_install(&argv[1], do_inst);
+		break;
+	case PKG_UPGRD_CMD: /* upgrade keep-packages */
+		pkgin_upgrade(UPGRADE_KEEP);
+		break;
+	case PKG_FUPGRD_CMD: /* upgrade everything installed */
+		pkgin_upgrade(UPGRADE_ALL);
+		break;
+	case PKG_REMV_CMD: /* remove packages and reverse dependencies */
+		missing_param(argc, 2, MSG_PKG_ARGS_RM);
+		pkgin_remove(&argv[1]);
+		break;
+	case PKG_AUTORM_CMD: /* autoremove orphan packages */
+		pkgin_autoremove();
+		break;
+	case PKG_KEEP_CMD: /* mark a package as "keep" (not automatic) */
+		missing_param(argc, 2, MSG_PKG_ARGS_KEEP);
+		pkg_keep(KEEP, &argv[1]);
+		break;
+	case PKG_UNKEEP_CMD: /* mark a package as "unkeep" (automatic) */
+		missing_param(argc, 2, MSG_PKG_ARGS_UNKEEP);
+		pkg_keep(UNKEEP, &argv[1]);
+		break;
+	case PKG_SHKP_CMD: /* show keep packages */
+		show_pkg_keep();
+		break;
+	case PKG_SRCH_CMD: /* search for package */
+		missing_param(argc, 2, MSG_MISSING_SRCH);
+		search_pkg(argv[1]);
+		break;
+	case PKG_CLEAN_CMD: /* clean pkgin's packages cache */
+		clean_cache();
+		break;
+	case PKG_EXPORT_CMD: /* export PKGPATH for keep packages */
+		export_keep();
+		break;
+	case PKG_IMPORT_CMD: /* import for keep packages and install them */
+		missing_param(argc, 2, MSG_MISSING_FILENAME);
+		import_keep(do_inst, argv[1]);
+		break;
+	case PKG_SHPROV_CMD: /* show what a package provides */
+		missing_param(argc, 2, MSG_MISSING_PKGNAME);
+		show_prov_req(GET_PROVIDES_QUERY, argv[1]);
+		break;
+	case PKG_SHREQ_CMD: /* show what a package requires */
+		missing_param(argc, 2, MSG_MISSING_PKGNAME);
+		show_prov_req(GET_REQUIRES_QUERY, argv[1]);
+		break;
+	case PKG_GINTO_CMD: /* Miod's request */
+		ginto();
+		break;
+	default:
+		usage();
+		/* NOTREACHED */
 	}
 
+	free_global_pkglists();
+
 	pkgindb_close();
 
+	if (tracefp != NULL)
+		fclose(tracefp);
+
 	XFREE(env_repos);
 	XFREE(pkg_repos);
 
@@ -250,45 +296,15 @@ usage()
 	printf(MSG_CMDS_SHORTCUTS);
 
 	for (i = 0; cmd[i].name != NULL; i++)
-		printf("%s (%s) -  %s\n", cmd[i].name, cmd[i].shortcut, cmd[i].descr);
+		if (cmd[i].cmdtype != PKG_GINTO_CMD)
+			printf("%s (%s) -  %s\n",
+				cmd[i].name, cmd[i].shortcut, cmd[i].descr);
 
 	exit(EXIT_FAILURE);
 }
 
 static void
-split_repos()
-{
-	int		repocount;
-	char	*p;
-
-	XSTRDUP(env_repos, getenv("PKG_REPOS"));
-
-	if (env_repos == NULL)
-		if ((env_repos = read_repos()) == NULL)
-			errx(EXIT_FAILURE, MSG_MISSING_PKG_REPOS);
-
-	repocount = 2; /* 1st elm + NULL */
-
-	XMALLOC(pkg_repos, repocount * sizeof(char *));
-	*pkg_repos = env_repos;
-
-	p = env_repos;
-
-	while((p = strchr(p, ' ')) != NULL) {
-		*p = '\0';
-		p++;
-
-		XREALLOC(pkg_repos, ++repocount * sizeof(char *));
-		pkg_repos[repocount - 2] = p;
-	}
-
-	/* NULL last element */
-	pkg_repos[repocount - 1] = NULL;
-
-	repo_record(pkg_repos);
-}
-
-void
-cleanup(int i)
+ginto()
 {
+	printf("* 2 oz gin\n* 5 oz tonic water\n* 1 lime wedge\n");
 }
