@@ -1,4 +1,4 @@
-/* $Id: messages.h,v 1.2 2011/04/03 16:17:08 imilh Exp $ */
+/* $Id: messages.h,v 1.24 2011/10/15 21:05:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -33,6 +33,7 @@
 /* main.c */
 #define MSG_MISSING_CMD "missing command"
 #define MSG_MISSING_PKGNAME "missing package name"
+#define MSG_MISSING_FILENAME "missing file name"
 #define MSG_FULLDEPTREE "full dependency tree for %s\n"
 #define MSG_REVDEPTREE "local reverse dependency tree for %s\n"
 #define MSG_PKG_ARGS_INST "specify at least one package to install"
@@ -41,7 +42,7 @@
 #define MSG_PKG_ARGS_UNKEEP "specify at least one package to unkeep"
 #define MSG_MISSING_SRCH "missing search string"
 
-#define MSG_USAGE "Usage: %s [-cdfFhlnPvVy] command [package ...]\n"
+#define MSG_USAGE "Usage: %s [-cdfFhlnPtvVy] command [package ...]\n"
 #define MSG_CMDS_SHORTCUTS "\nCommands and shortcuts:\n"
 
 #define MSG_CHROOT_FAILED "Unable to chroot"
@@ -49,23 +50,28 @@
 
 #define MSG_MISSING_PKG_REPOS \
 	PKGIN_CONF"/"REPOS_FILE" has no repositories or does not exist.\nNo PKG_REPOS variable to fallback to."
+#define MSG_CANT_OPEN_WRITE "Couldn't open %s for writing.\n"
+#define MSG_PKG_INSTALL_NOT_PRESENT "pkg_install package is not installed."
 
 /* actions.c */
-#define MSG_PROCEED "proceed ? [y/N] "
+#define MSG_PROCEED_YES "proceed ? [Y/n] "
+#define MSG_PROCEED_NO "proceed ? [y/N] "
 #define MSG_REMOVING "removing %s...\n"
 #define MSG_DOWNLOAD_PKGS "downloading packages...\n"
 #define MSG_PKG_NO_REPO "%s has no associated repository"
+#define MSG_SYMLINK_FAILED "failed to create symlink %s"
+#define MSG_SYMLINKING_PKG "symlinking %s...\n"
 #define MSG_ERR_OPEN "error opening %s"
 #define MSG_INSTALL_PKG "installing packages...\n"
 #define MSG_INSTALLING "installing %s...\n"
 #define MSG_REQT_NOT_PRESENT \
 	"%s, needed by %s is not present in this system.\n"
-#define MSG_REQT_NOT_PRESENT_BUT \
-	"warning: %s is not present in this system (may be installed by this package)\n"
+#define MSG_REQT_NOT_PRESENT_DEPS \
+	"warning: %s is not present in this system nor package's dependencies\n"
 #define MSG_CONFLICT_PKG \
 	"%s (to be installed) conflicts with installed package %s.\n"
 #define MSG_NOTHING_TO_DO "nothing to do.\n"
-#define MSG_REQT_MISSING "required files are missing in this system.\n"
+#define MSG_REQT_MISSING "the following packages have unmet requirements: %s\n\n"
 #define MSG_NO_CACHE_SPACE "%s has not enough space for download\n"
 #define MSG_NO_INSTALL_SPACE "%s has not enough space for installation\n"
 #define MSG_PKGS_TO_UPGRADE "%d packages to be upgraded: %s\n"
@@ -82,10 +88,13 @@
 #define MSG_EMPTY_KEEP_LIST "empty non-autoremovable package list"
 #define MSG_EMPTY_AVAIL_PKGLIST "empty available packages list"
 #define MSG_UPGRADE_NEED_REMOVE "this upgrade needs package removal\n"
-#define MSG_LOGGING_TO "error log can be found in %s\n"
+#define MSG_PKG_INSTALL_LOGGING_TO "pkg_install error log can be found in %s\n"
 #define MSG_EMPTY_FILE_SIZE "warning: package %s has an empty FILE_SIZE on repository.\n"
 #define MSG_NOT_REMOVING "not removing %s...\n"
 #define MSG_UPGRADE_PKG_INSTALL "about to upgrade %s, "
+#define MSG_ERR_INSTALLING_PKG "/!\\ there was an error while installing %s, please check %s\n"
+#define MSG_ERR_REMOVING_PKG "/!\\ there was an error while removing %s, please check %s\n"
+#define MSG_WARNS_ERRS "pkg_install warnings: %d, errors: %d\n"
 
 /* depends.c */
 #define MSG_DIRECT_DEPS_FOR "direct dependencies for %s\n"
@@ -96,13 +105,14 @@
 
 /* autoremove.c */
 #define MSG_AUTOREMOVE_WARNING "\
-in order to remove packages from the autoremove list, flag those with the -k modifier.\n"
+in order to remove packages from the autoremove list, flag those with the `keep' action.\n"
 #define MSG_NO_PKGIN_PKGS "no packages have been installed with %s"
-#define MSG_ALL_KEEP_PKGS "all packages are marked as \"keepable\"."
+#define MSG_ALL_KEEP_PKGS "all packages are marked as \"keepable\"\n."
 #define MSG_AUTOREMOVE_PKGS "%d packages to be autoremoved: %s\n"
 #define MSG_MARK_PKG_KEEP "%s is marked as non-autoremovable\n"
 #define MSG_MARKING_PKG_KEEP "marking %s as non auto-removable\n"
 #define MSG_UNMARKING_PKG_KEEP "marking %s as auto-removable\n"
+#define MSG_NO_ORPHAN_DEPS "no orphan dependencies found.\n"
 
 /* summary.c */
 #define MSG_UPDATING_DB "updating database:   0%%"
@@ -113,6 +123,8 @@ in order to remove packages from the autoremove list, flag those with the -k mod
 #define MSG_PROCESSING_REMOTE_SUMMARY "processing remote summary (%s)...\n"
 #define MSG_COULDNT_FETCH "Could not fetch %s\n"
 #define MSG_ARCH_DONT_MATCH "\r\n/!\\ Warning /!\\ %s doesn't match your current architecture (%s)\nYou probably want to modify "PKGIN_CONF"/"REPOS_FILE".\nStill want to "
+#define MSG_COULD_NOT_GET_PKGNAME "Could not get package name from dependency: %s\n"
+#define MSG_DATABASE_NOT_COMPAT "Database needs to be updated.\n"
 
 /* impact.c */
 #define MSG_GREATER_VERSION \
@@ -121,11 +133,11 @@ in order to remove packages from the autoremove list, flag those with the -k mod
 	"there's more than one version available for this package.\n\
 please re-run %s with a package name matching one of the following:\n"
 #ifndef DEBUG
-#define MSG_PKG_NOT_AVAIL "%s is not available on the repository\n"
+#define MSG_CALCULATING_DEPS "\rcalculating dependencies..."
 #else
 #define MSG_CALCULATING_DEPS "calculating dependencies for %s...\n"
 #endif
-#define MSG_CALCULATING_DEPS "\rcalculating dependencies..."
+#define MSG_PKG_NOT_AVAIL "%s is not available on the repository\n"
 #define MSG_BROKEN_DEP "%s has no dependency in pkg_summary(5), while it's a reverse dependency for %s (missing package in repository ?). Default behaviour is to remove %s. "
 
 /* pkglist.c */
@@ -135,3 +147,10 @@ please re-run %s with a package name matching one of the following:\n"
 /* fsops.c */
 #define MSG_TRANS_FAILED "Failed to translate %s in repository config file"
 #define MSG_INVALID_REPOS "Invalid repository: %s"
+
+/* selection.c */
+#define MSG_EMPTY_IMPORT_LIST "Empty import list."
+
+/* pkg_check.c */
+#define MSG_NO_PROV_REQ "Nothing %s by %s.\n"
+#define MSG_FILES_PROV_REQ "Files %s by %s:\n"
