@@ -1,4 +1,4 @@
-/* $Id: pkgin.h,v 1.3 2011/08/09 11:38:34 imilh Exp $ */
+/* $Id: pkgin.h,v 1.25 2011/10/10 13:06:37 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -45,6 +45,11 @@
 #include "pkgindb.h"
 #include "tools.h"
 #include "lib.h"
+#include "dewey.h"
+
+#ifndef __UNCONST
+#define __UNCONST(a)	((void *)(unsigned long)(const void *)(a))
+#endif
 
 #ifndef PKGTOOLS
 #define PKGTOOLS "/usr/sbin"
@@ -54,7 +59,7 @@
 
 #define PKG_SUMMARY "pkg_summary"
 #define PKGIN_SQL_LOG PKGIN_DB"/sql.log"
-#define PKGIN_ERR_LOG PKGIN_DB"/err.log"
+#define PKG_INSTALL_ERR_LOG PKGIN_DB"/pkg_install-err.log"
 #define PKGIN_CACHE PKGIN_DB"/cache"
 #define PKG_EXT ".tgz"
 #define PKGIN_CONF PKG_SYSCONFDIR"/pkgin"
@@ -68,6 +73,7 @@
 #define TOINSTALL 0
 #define TOUPGRADE 1
 #define TOREMOVE 2
+#define UNMET_REQ 3
 
 #define KEEP 1
 #define UNKEEP 0
@@ -99,48 +105,85 @@
 #define PKG_SRCH_CMD 13
 #define PKG_CLEAN_CMD 14
 #define PKG_AUTORM_CMD 15
+#define PKG_EXPORT_CMD 16
+#define PKG_IMPORT_CMD 17
+#define PKG_SHPROV_CMD 18
+#define PKG_SHREQ_CMD 19
+#define PKG_GINTO_CMD 255
 
 #define PKG_EQUAL '='
 #define PKG_GREATER '>'
 #define PKG_LESSER '<'
 
+#define DEFAULT_NO 0
+#define DEFAULT_YES 1
+#define ANSW_NO 0
+#define ANSW_YES 1
+
+#define TRACE(fmt...) if (tracefp != NULL) fprintf(tracefp, fmt)
+
 typedef struct Dlfile {
 	char *buf;
 	size_t size;
 } Dlfile;
 
-typedef struct Pkgdeptree {
-	char *depname; /* foo>=1.0 */
-	char *matchname; /* foo */
-	int computed; /* recursion memory */
-	int level; /* recursion level */
-	int pkgkeep; /* autoremovable package ? */
-	int64_t file_size; /* binary package size, used in order.c and actions.c */
-	SLIST_ENTRY(Pkgdeptree) next;
-} Pkgdeptree;
+/**
+ * \struct Deptree
+ * \brief Package dependency tree
+ */
+typedef struct Deptree {
+	int		computed; /*!< recursion memory */
+	int		keep; /*!< autoremovable package ? */
+} Deptree;
 
+/**
+ * \struct Impact
+ * \brief Impact list
+ */
+typedef struct Pkgimpact {
+	int		action; /*!< TOINSTALL or TOUPGRADE */
+	char	*old; /*!< package to upgrade: perl-5.8 */
+} Impact;
+
+/**
+ * \struct Pkglist
+ *
+ * \brief Master structure for all types of package lists (SLIST)
+ */
 typedef struct Pkglist {
-	char *pkgname; /* foo-1.0 */
-	char *comment; /* comment */
-	int64_t file_size; /* binary package size */
-	int64_t size_pkg; /* installed package size */
+	uint8_t	type; /*!< list type (LIST, DEPTREE or IMPACT) */
+
+	int64_t	size_pkg; /*!< installed package size (list and impact) */
+	int64_t	file_size; /*!< binary package size */
+	int		level; /*<! recursion level (deptree and impact) */
+
+	char *full; /*!< full package name with version, foo-1.0 */
+	char *name; /*!< package name, foo */
+	char *version; /*<! package version, 1.0 */
+	char *depend; /*!< dewey or glob form for forward (direct) dependencies:
+					foo>=1.0
+					or full package name for reverse dependencies:
+					foo-1.0 */
+	union {
+		char		*comment; /*!< package list comment */
+		Deptree		deptree; /*<! dependency tree informations */
+		Impact		impact; /*<! impact list informations */
+	} p_un;
+
 	SLIST_ENTRY(Pkglist) next;
 } Pkglist;
 
-typedef struct Pkgimpact {
-	char *depname; /* depencendy pattern: perl-[0-9]* */
-	char *pkgname; /* real dependency name: perl-5.10 */
-	char *oldpkg; /* package to upgrade: perl-5.8 */
-	int action; /* TOINSTALL or TOUPGRADE */
-	int level; /* dependency level, inherited from full dependency list */
-	int64_t file_size; /* binary package size */
-	int64_t size_pkg; /* installed package size */
-	SLIST_ENTRY(Pkgimpact) next;
-} Pkgimpact;
+#define comment  	p_un.comment
+#define computed	p_un.deptree.computed
+#define keep		p_un.deptree.keep
+#define old		   	p_un.impact.old
+#define action   	p_un.impact.action
+
+#define LIST		0
+#define DEPTREE		1
+#define IMPACT		2
 
-typedef SLIST_HEAD(, Pkgdeptree) Deptreehead;
 typedef SLIST_HEAD(, Pkglist) Plisthead;
-typedef SLIST_HEAD(, Pkgimpact) Impacthead;
 
 extern uint8_t 		yesflag;
 extern uint8_t 		noflag;
@@ -148,46 +191,53 @@ extern uint8_t 		force_update;
 extern uint8_t 		force_reinstall;
 extern uint8_t		verbosity;
 extern uint8_t		package_version;
+extern uint8_t		pi_upgrade; /* pkg_install upgrade */
+extern char			*env_repos;
 extern char			**pkg_repos;
 extern const char	*pkgin_cache;
 extern char  		lslimit;
 extern char			pkgtools_flags[];
+extern Plisthead	r_plisthead;
+extern Plisthead	l_plisthead;
+extern FILE			*tracefp;
 
 /* download.c*/
 Dlfile		*download_file(char *, time_t *);
 /* summary.c */
 void		update_db(int, char **);
+void		split_repos(void);
+/* sqlite_callbacks.c */
+int			pdb_rec_list(void *, int, char **, char **);
+int			pdb_rec_depends(void *, int, char **, char **);
 /* depends.c */
-int			exact_pkgfmt(const char *);
-char 		*match_dep_ext(char *, const char *);
 void		show_direct_depends(const char *);
 void		show_full_dep_tree(const char *, const char *, const char *);
 void 		full_dep_tree(const char *pkgname, const char *depquery,
-	Deptreehead	*pdphead);
-void		free_deptree(Deptreehead *);
+	Plisthead	*pdphead);
 /* pkglist.c */
-Plisthead	*rec_pkglist(const char *);
-void		free_pkglist(Plisthead *);
+void		init_global_pkglists(void);
+void		free_global_pkglists(void);
+Pkglist		*malloc_pkglist(uint8_t);
+void		free_pkglist_entry(Pkglist **, uint8_t);
+void		free_pkglist(Plisthead **, uint8_t);
+Plisthead	*init_head(void);
+Plisthead	*rec_pkglist(const char *, ...);
 void		list_pkgs(const char *, int);
 void		search_pkg(const char *);
-char		*find_exact_pkg(Plisthead *, const char *);
-char		*end_expr(Plisthead *, const char *);
 /* actions.c */
-int			check_yesno(void);
+int			check_yesno(uint8_t);
+void		do_pkg_remove(Plisthead *);
 int			pkgin_remove(char **);
 int			pkgin_install(char **, uint8_t);
 char		*action_list(char *, char *);
 void		pkgin_upgrade(int);
 /* order.c */
-Deptreehead	*order_remove(Deptreehead *);
-Deptreehead	*order_upgrade_remove(Impacthead *);
-Deptreehead	*order_install(Impacthead *);
+Plisthead	*order_remove(Plisthead *);
+Plisthead	*order_upgrade_remove(Plisthead *);
+Plisthead	*order_install(Plisthead *);
 /* impact.c */
-Impacthead	*pkg_impact(char **);
-void		free_impact(Impacthead *impacthead);
-Pkglist		*map_pkg_to_dep(Plisthead *, char *);
-int			count_samepkg(Plisthead *, const char *);
-int			version_check(char *, char *);
+uint8_t		pkg_in_impact(Plisthead *, char *);
+Plisthead	*pkg_impact(char **);
 /* autoremove.c */
 void	   	pkgin_autoremove(void);
 void		show_pkg_keep(void);
@@ -197,5 +247,21 @@ int			fs_has_room(const char *, int64_t);
 void		clean_cache(void);
 void		create_dirs(void);
 char		*read_repos(void);
+/* pkg_str.c */
+char	   	*unique_pkg(const char *, const char *);
+Pkglist		*map_pkg_to_dep(Plisthead *, char *);
+uint8_t		non_trivial_glob(char *);
+char		*get_pkgname_from_depend(char *);
+int			exact_pkgfmt(const char *);
+char		*find_exact_pkg(Plisthead *, const char *);
+int			version_check(char *, char *);
+char		**glob_to_pkgarg(char **);
+/* selection.c */
+void		export_keep(void);
+void		import_keep(uint8_t, const char *);
+/* pkg_check.c */
+int			pkg_met_reqs(Plisthead *);
+int			pkg_has_conflicts(Pkglist *);
+void		show_prov_req(const char *, const char *);
 
 #endif
