@@ -1,4 +1,4 @@
-/* $Id: pkgindb.c,v 1.1.1.1 2011/03/03 14:43:13 imilh Exp $ */
+/* $Id: pkgindb.c,v 1.8 2011/09/11 10:51:37 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -36,7 +36,7 @@
 static sqlite3	*pdb;
 static char		*pdberr = NULL;
 static int		pdbres = 0;
-static FILE		*fp;
+static FILE		*sql_log_fp;
 
 static const char *pragmaopts[] = {
 	"cache_size = 1000000",
@@ -78,6 +78,9 @@ pkgindb_simple_callback(void *param, int argc, char **argv, char **colname)
 {
 	pdbres = argc;
 
+	if (argv == NULL)
+		return PDB_ERR;
+
 	return PDB_OK;
 }
 
@@ -89,6 +92,7 @@ pdb_get_value(void *param, int argc, char **argv, char **colname)
 
 	if (argv != NULL) {
 		XSTRCPY(value, argv[0]);
+
 		return PDB_OK;
 	}
 
@@ -101,10 +105,10 @@ pkgindb_doquery(const char *query,
 {
 	if (sqlite3_exec(pdb, query, pkgindb_callback, param, &pdberr)
 		!= SQLITE_OK) {
-		if (fp != NULL) {
+		if (sql_log_fp != NULL) {
 			if (pdberr != NULL)
-				fprintf(fp, "SQL error: %s\n", pdberr);
-			fprintf(fp, "SQL query: %s\n", query);
+				fprintf(sql_log_fp, "SQL error: %s\n", pdberr);
+			fprintf(sql_log_fp, "SQL query: %s\n", query);
 		}
 		sqlite3_free(pdberr);
 
@@ -119,8 +123,26 @@ pkgindb_close()
 {
 	sqlite3_close(pdb);
 
-	if (fp != NULL)
-		fclose(fp);
+	if (sql_log_fp != NULL)
+		fclose(sql_log_fp);
+}
+
+uint8_t
+upgrade_database()
+{
+	if (pkgindb_doquery(COMPAT_CHECK,
+			pkgindb_simple_callback, NULL) == PDB_ERR) {
+		/* COMPAT_CHECK query leads to an error for an incompatible database */
+		printf(MSG_DATABASE_NOT_COMPAT);
+		if (!check_yesno(DEFAULT_YES))
+			exit(EXIT_FAILURE);
+
+		pkgindb_reset();
+
+		return 1;
+	}
+
+	return 0;
 }
 
 void
@@ -133,7 +155,7 @@ pkgindb_init()
 	 * Do not exit if PKGIN_SQL_LOG is not writable.
 	 * Permit users to do list-operations
 	 */
-	fp = fopen(PKGIN_SQL_LOG, "w");
+	sql_log_fp = fopen(PKGIN_SQL_LOG, "w");
 
 	if (sqlite3_open(PDB, &pdb) != SQLITE_OK)
 		pdb_err("Can't open database " PDB);
@@ -152,22 +174,34 @@ pkgindb_init()
 	pkgindb_doquery(CREATE_DRYDB, NULL, NULL);
 }
 
+/**
+ * \brief destroy the database and re-create it (upgrade)
+ */
+void
+pkgindb_reset()
+{
+	pkgindb_close();
+
+	if (unlink(PDB) < 0)
+		err(EXIT_FAILURE, "could not delete database file %s\n", PDB);
+
+	pkgindb_init();
+}
+
 #define PKGDB_PATH PKG_DBDIR"/pkgdb.byfile.db"
 
 int
 pkg_db_mtime()
 {
+	uint8_t		pkgdb_present = 1;
 	struct stat	st;
 	time_t	   	db_mtime = 0;
-	char		*str_mtime, query[BUFSIZ];
+	char		str_mtime[20], query[BUFSIZ];
 
 	/* no pkgdb file */
-	if (stat(PKGDB_PATH, &st) < 0) {
-		warnx("%s does not exist.", PKGDB_PATH);
-		return 1;
-	}
+	if (stat(PKGDB_PATH, &st) < 0)
+		pkgdb_present = 0;
 
-	XMALLOC(str_mtime, 20 * sizeof(char));
 	str_mtime[0] = '\0';
 
 	pkgindb_doquery("SELECT PKGDB_MTIME FROM PKGDB;",
@@ -176,10 +210,8 @@ pkg_db_mtime()
 	if (str_mtime[0] != '\0')
 		db_mtime = (time_t)strtol(str_mtime, (char **)NULL, 10);
 
-	XFREE(str_mtime);
-
 	/* mtime is up to date */
-	if (db_mtime == st.st_mtime)
+	if (!pkgdb_present || db_mtime == st.st_mtime)
 		return 0;
 
 	snprintf(query, BUFSIZ, UPDATE_PKGDB_MTIME, (long long)st.st_mtime);
@@ -211,9 +243,8 @@ time_t
 pkg_sum_mtime(char *repo)
 {
 	time_t	db_mtime = 0;
-	char	*str_mtime, query[BUFSIZ];
+	char	str_mtime[20], query[BUFSIZ];
 
-	XMALLOC(str_mtime, 20 * sizeof(char));
 	str_mtime[0] = '\0';
 
 	snprintf(query, BUFSIZ,
@@ -223,7 +254,5 @@ pkg_sum_mtime(char *repo)
 	if (str_mtime[0] != '\0')
 		db_mtime = (time_t)strtol(str_mtime, (char **)NULL, 10);
 
-	XFREE(str_mtime);
-
 	return db_mtime;
 }
