@@ -1,4 +1,4 @@
-/* $Id: pkgindb.h,v 1.1.1.1 2011/03/03 14:43:13 imilh Exp $ */
+/* $Id: pkgindb.h,v 1.8 2011/09/11 08:39:16 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -33,6 +33,7 @@
 #ifndef _DRYDB_H
 #define _DRYDB_H
 
+#include <stdint.h>
 #include "pkgindb_create.h"
 
 extern const char DROP_LOCAL_TABLES[];
@@ -60,10 +61,22 @@ extern const char UPDATE_PKGDB_MTIME[];
 extern const char EXISTS_REPO[];
 extern const char INSERT_REPO[];
 extern const char UPDATE_REPO_MTIME[];
+extern const char INSERT_SINGLE_VALUE[];
+extern const char INSERT_DEPENDS_VALUES[];
+extern const char UNIQUE_EXACT_PKG[];
+extern const char UNIQUE_PKG[];
+extern const char EXPORT_KEEP_LIST[];
+extern const char GET_PKGNAME_BY_PKGPATH[];
+extern const char GET_ORPHAN_PACKAGES[];
+extern const char COMPAT_CHECK[];
+
+#define LOCAL_PKG "LOCAL_PKG"
+#define REMOTE_PKG "REMOTE_PKG"
 
 #define PDB PKGIN_DB"/pkgin.db"
 
 const char	*pdb_version(void);
+uint8_t		upgrade_database(void);
 void		pkgindb_init(void);
 void		pkgindb_close(void);
 int			pkgindb_doquery(const char *,
@@ -72,6 +85,7 @@ int			pdb_get_value(void *, int, char **, char **);
 int			pkg_db_mtime(void);
 void		repo_record(char **);
 time_t		pkg_sum_mtime(char *);
+void		pkgindb_reset(void);
 
 #define PDB_OK 0
 #define PDB_ERR -1
