@@ -1,4 +1,4 @@
-/* $Id: pkgindb_queries.c,v 1.1.1.1 2011/03/03 14:43:13 imilh Exp $ */
+/* $Id: pkgindb_queries.c,v 1.15 2011/10/04 08:17:36 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -57,36 +57,35 @@ const char DELETE_REMOTE[] =
     "WHERE REMOTE_PKG.REPOSITORY GLOB '%s*' AND "
     "REMOTE_PKG.PKG_ID = %s.PKG_ID);";
 
-const char DIRECT_DEPS[] =
-    "SELECT REMOTE_DEPS.REMOTE_DEPS_PKGNAME "
-    "FROM REMOTE_DEPS,REMOTE_PKG "
-    "WHERE REMOTE_PKG.PKGNAME GLOB \'%s-[0-9]*\' "
-    "AND REMOTE_DEPS.PKG_ID = REMOTE_PKG.PKG_ID;";
+const char DIRECT_DEPS[] = /* prefer higher version */
+	"SELECT REMOTE_DEPS_DEWEY, REMOTE_DEPS_PKGNAME "
+	"FROM REMOTE_DEPS WHERE PKG_ID = "
+	"(SELECT PKG_ID FROM REMOTE_PKG WHERE PKGNAME = '%s' "
+	"ORDER BY FULLPKGNAME DESC LIMIT 1);";
 
 const char LOCAL_DIRECT_DEPS[] =
-    "SELECT LOCAL_DEPS.LOCAL_DEPS_PKGNAME "
-    "FROM LOCAL_DEPS,LOCAL_PKG "
-    "WHERE LOCAL_PKG.PKGNAME GLOB \'%s-[0-9]*\' "
-    "AND LOCAL_DEPS.PKG_ID = LOCAL_PKG.PKG_ID;";
+	"SELECT LOCAL_DEPS_DEWEY, LOCAL_DEPS_PKGNAME "
+	"FROM LOCAL_DEPS WHERE PKG_ID = "
+	"(SELECT PKG_ID FROM LOCAL_PKG WHERE PKGNAME = '%s' "
+	"ORDER BY FULLPKGNAME DESC LIMIT 1);";
 
 const char EXACT_DIRECT_DEPS[] =
-    "SELECT REMOTE_DEPS.REMOTE_DEPS_PKGNAME "
-    "FROM REMOTE_DEPS,REMOTE_PKG "
-    "WHERE REMOTE_PKG.PKGNAME GLOB \'%s*\' "
-    "AND REMOTE_DEPS.PKG_ID = REMOTE_PKG.PKG_ID;";
+	"SELECT REMOTE_DEPS.REMOTE_DEPS_DEWEY, REMOTE_DEPS.REMOTE_DEPS_PKGNAME "
+	"FROM REMOTE_DEPS,REMOTE_PKG "
+	"WHERE REMOTE_PKG.FULLPKGNAME = '%s' "
+	"AND REMOTE_DEPS.PKG_ID = REMOTE_PKG.PKG_ID;";
 
-const char const LOCAL_REVERSE_DEPS[] =
-    "SELECT LOCAL_PKG.PKGNAME,LOCAL_PKG.PKG_KEEP "
+const char LOCAL_REVERSE_DEPS[] =
+    "SELECT LOCAL_PKG.FULLPKGNAME, LOCAL_PKG.PKGNAME, LOCAL_PKG.PKG_KEEP "
     "FROM LOCAL_PKG,LOCAL_DEPS "
-    "WHERE LOCAL_DEPS.LOCAL_DEPS_PKGNAME "
-    "GLOB \'%s[<>\\[={*-][^a-zA-Z]*\' "
+	"WHERE LOCAL_DEPS.LOCAL_DEPS_PKGNAME  = '%s' "
     "AND LOCAL_PKG.PKG_ID = LOCAL_DEPS.PKG_ID;";
 
 const char LOCAL_CONFLICTS[] =
     "SELECT LOCAL_CONFLICTS_PKGNAME FROM LOCAL_CONFLICTS;";
 
 const char GET_CONFLICT_QUERY[] =
-    "SELECT LOCAL_PKG.PKGNAME  FROM LOCAL_CONFLICTS,LOCAL_PKG "
+    "SELECT LOCAL_PKG.FULLPKGNAME FROM LOCAL_CONFLICTS,LOCAL_PKG "
     "WHERE LOCAL_CONFLICTS.LOCAL_CONFLICTS_PKGNAME = '%s' "
     "AND LOCAL_CONFLICTS.PKG_ID = LOCAL_PKG.PKG_ID;";
 
@@ -96,13 +95,13 @@ const char GET_CONFLICT_QUERY[] =
 const char GET_REQUIRES_QUERY[] =
     "SELECT REMOTE_REQUIRES.REMOTE_REQUIRES_PKGNAME "
     "FROM REMOTE_REQUIRES,REMOTE_PKG "
-    "WHERE REMOTE_PKG.PKGNAME = \'%s\' "
+    "WHERE REMOTE_PKG.FULLPKGNAME = '%s' "
     "AND REMOTE_REQUIRES.PKG_ID = REMOTE_PKG.PKG_ID;";
 
 const char GET_PROVIDES_QUERY[] =
     "SELECT REMOTE_PROVIDES.REMOTE_PROVIDES_PKGNAME "
     "FROM REMOTE_PROVIDES,REMOTE_PKG "
-    "WHERE REMOTE_PKG.PKGNAME = \'%s\' "
+    "WHERE REMOTE_PKG.FULLPKGNAME = '%s' "
     "AND REMOTE_PROVIDES.PKG_ID = REMOTE_PKG.PKG_ID;";
 
 const char LOCAL_PROVIDES[] =
@@ -114,21 +113,23 @@ const char UNKEEP_PKG[] =
     "UPDATE LOCAL_PKG SET PKG_KEEP = NULL WHERE PKGNAME = \'%s\';";
 
 const char LOCAL_PKGS_QUERY[] =
-    "SELECT PKGNAME,COMMENT,FILE_SIZE,SIZE_PKG FROM LOCAL_PKG "
-    "ORDER BY PKGNAME DESC;";
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,COMMENT,FILE_SIZE,SIZE_PKG "
+	"FROM LOCAL_PKG "
+    "ORDER BY FULLPKGNAME ASC;";
 
 const char REMOTE_PKGS_QUERY[] =
-    "SELECT PKGNAME,COMMENT,FILE_SIZE,SIZE_PKG FROM REMOTE_PKG "
-    "ORDER BY PKGNAME DESC;";
+    "SELECT FULLPKGNAME,PKGNAME,PKGVERS,COMMENT,FILE_SIZE,SIZE_PKG "
+	"FROM REMOTE_PKG "
+    "ORDER BY FULLPKGNAME ASC;";
 
 const char NOKEEP_LOCAL_PKGS[] =
-    "SELECT PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NULL;";
+    "SELECT FULLPKGNAME,PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NULL;";
 
 const char KEEP_LOCAL_PKGS[] =
-    "SELECT PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NOT NULL;";
+    "SELECT FULLPKGNAME,PKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NOT NULL;";
 
 const char PKG_URL[] =
-    "SELECT REPOSITORY FROM REMOTE_PKG WHERE PKGNAME = \'%s\';";
+    "SELECT REPOSITORY FROM REMOTE_PKG WHERE FULLPKGNAME = \'%s\';";
 
 const char DELETE_EMPTY_ROWS[] =
     "DELETE FROM REMOTE_PKG WHERE PKGNAME IS NULL;";
@@ -144,3 +145,31 @@ const char INSERT_REPO[] =
 
 const char UPDATE_REPO_MTIME[] =
     "UPDATE REPOS SET REPO_MTIME = %lld WHERE REPO_URL = \'%s\';";
+
+const char INSERT_SINGLE_VALUE[] =
+	"INSERT INTO %s (PKG_ID, %s_PKGNAME) VALUES (%d,\"%s\");";
+
+const char INSERT_DEPENDS_VALUES[] = 
+	"INSERT INTO %s (PKG_ID, %s_PKGNAME, %s_DEWEY) VALUES (%d,\"%s\",\"%s\");";
+
+const char UNIQUE_PKG[] = 
+	"SELECT FULLPKGNAME FROM %s WHERE PKGNAME = '%s' "
+	"ORDER BY PKGVERS DESC LIMIT 1;";
+
+const char UNIQUE_EXACT_PKG[] = 
+	"SELECT FULLPKGNAME FROM %s WHERE FULLPKGNAME GLOB '%s*' "
+	"ORDER BY PKGVERS DESC LIMIT 1;";
+
+const char EXPORT_KEEP_LIST[] =
+	"SELECT PKGPATH FROM LOCAL_PKG WHERE PKG_KEEP IS NOT NULL "
+	"ORDER BY PKG_ID DESC;";
+
+const char GET_PKGNAME_BY_PKGPATH[] = 
+	"SELECT PKGNAME FROM REMOTE_PKG WHERE PKGPATH = '%s';";
+
+const char GET_ORPHAN_PACKAGES[] = 
+	"SELECT FULLPKGNAME FROM LOCAL_PKG WHERE PKG_KEEP IS NULL AND "
+	"PKGNAME NOT IN (SELECT LOCAL_DEPS_PKGNAME FROM LOCAL_DEPS);";
+
+const char COMPAT_CHECK[] =
+	"SELECT FULLPKGNAME FROM REMOTE_PKG LIMIT 1;";
